# -*- coding: utf-8 -*-

"""
Module de géométrie utilisé par le script BillardVersionFonctions
@author: Loïc Devilliers
"""

import numpy as np
import matplotlib.pyplot as plt#Bibliothèque Pyton qui gère l'affichage

def DroiteProlongeSegment(S):

    """
    Renvoie une équation de la droite passant par les 2 points du segment S.
    Une équation de droite est sous la forme ax+by=c cette fonction
    renvoie (a,b,c).
    """
    assert(S[0]!=S[1]), "Il faut deux points distincts pour définir une droite"
    M=S[0]
    N=S[1]
    return (M[1]-N[1],N[0]-M[0],-Det(M,N))

def IntersectionDeuxDroites(P1,P2):
    """
    Pour deux droites différentes du plan, renvoie les coordonnées
    de l'intersection en résolvant le système
    """
    d=Det(P1[:2],P2[:2])
    if d==0:
        return None#Les deux droites sont parallèles
    else:
        return (Det(P2[1:],P1[1:])/d,Det([P1[0],P1[2]],[P2[0],P2[2]])/d)

def Det(P1,P2):
    return P1[0]*P2[1]-P2[0]*P1[1]

def EtreDansSegment(S,P3):
    """
    Pour P3 un point de la droite prolongent le Segment S
    renvoie le booléen True si et seulement si P3 est dans le segment S
    """
    assert(P3!=S[0] and P3!=S[1]), "Il faut éviter de rebondir sur un sommet"
    P1=S[0]
    P2=S[1]
    if abs(P1[0]-P2[0])>abs(P1[1]-P2[1]):#La droite est plus proche d'être horizontale que verticale
        return (min(P1[0],P2[0])< P3[0]<max(P1[0],P2[0]))
    else:
        return (min(P1[1],P2[1])< P3[1]<max(P1[1],P2[1]))

def SymetriqueDUneDroiteParRapportAUneDroite(D,Delta):
    """
    Soit D' la droite symétrique à D par rapport à Delta
    Cette fonction renvoit l'équation de la droite D'.
    """
    [a,b,c]=Delta
    [u,v,w]=D
    Nc=a**2+b**2
    Up=u-2*(u*a**2+v*b*a)/Nc
    Vp=v-2*(v*b**2+u*b*a)/Nc
    Wp=w-2*c*(a*u+b*v)/Nc
    return (Up,Vp,Wp)

def TraceDroite(D):
    a=D[0]
    b=D[1]
    c=D[2]
    x=np.linspace(-5,5,100)
    return plt.plot(x, -(a*x-c)/b)

def Polygone(n,t=1):
    """
    Renvoie un polygone régulier à n coté si t=1, sinon renvoie
    ce même polygône mais étiré sur l'axe des abscisses avec le paramètre t
    Polygone(20) est une approximation d'un cercle de centre 0 et de rayon 1
    Polygone(20,2) est une approximation d'une ellipse de petit demi axe 1
    et de demi grand axe 2
    """
    return [(t*np.cos(2*k*np.pi/n),np.sin(2*k*np.pi/n)) for k in range(n)]

#Zone de tests du module
if __name__=="__main__":
    A=(1,2)
    B=(-1,3)
    C=(0,1)
    F=(2,1.5)
    D=DroiteProlongeSegment([A,B])
    Delta=DroiteProlongeSegment([A,C])
    S=SymetriqueDUneDroiteParRapportAUneDroite(D,Delta)
    plt.figure()
    TraceDroite(D)
    TraceDroite(Delta)
    TraceDroite(S)
    plt.axis('equal')
    plt.axis([-5,5,0,5])
    plt.title('D est la droite passant par A et B, S la droite symétrique à D'\
    '\n par rapport à la droite $\Delta$')
    E=IntersectionDeuxDroites(D,S)#E doit valoir A
    print("L'intersection entre D et S doit être A, est-ce le cas? ", E==A)
    plt.plot([A[0]],[A[1]],'o')
    plt.plot([B[0]],[B[1]],'o')
    plt.plot([F[0]],[F[1]],'o')
    plt.legend(['D','$\Delta$','S','A','B','F'])
    print("F est dans le segment [A,B]? ", EtreDansSegment([A,B],F))
    print("A est dans le segment [B,F]? ", EtreDansSegment([B,F],A))
    plt.show()